from linkpreview.preview.base import PreviewBase
from linkpreview.helpers import LazyAttribute

WEBSITE = "website"
ORGANIZATION = "organization"
BLOG = "blog"
site_types = (WEBSITE, BLOG, ORGANIZATION)


class SchemaPreviewBase(PreviewBase):
    """
    Schema.org meta properties
    """

    schema_netloc = "schema.org"

    def get_schema(self):  # pragma:nocover
        raise NotImplementedError

    @LazyAttribute
    def schema(self):
        return tuple(self.get_schema())

    @LazyAttribute
    def sorted_schema(self):
        return sorted(self.schema, key=lambda x: x["type"] in site_types)

    @property
    def site_name(self):
        for item in self.schema:
            if item["type"] not in site_types:
                continue

            if "name" in item:
                return item["name"]

    @property
    def title(self):
        for item in self.sorted_schema:
            if "name" in item:
                return item["name"]

    @property
    def description(self):
        for item in self.sorted_schema:
            if "description" in item:
                return item["description"]

    @property
    def image(self):
        for item in self.sorted_schema:
            if "image" in item:
                return item["image"]

            if "thumbnailUrl" in item:
                return item["thumbnailUrl"]



from linkpreview import link_preview

from tests.helpers import get_sample


def test_favicon():
    baseurl = "http://example.com"

    # absolute favicon
    preview = link_preview(
        url=baseurl,
        content=get_sample("favicon/absolute.html"),
    )
    assert preview.favicon == (
        ("http://thesite.com/favicon.ico", None, "shortcut icon"),
    )
    assert preview.absolute_favicon == (
        (preview.favicon[0][0], None, "shortcut icon"),
    )

    # relative favicon from root
    preview = link_preview(
        url=baseurl,
        content=get_sample("favicon/relative.html"),
    )
    assert preview.favicon == (("/path/to/favicon.ico", None, "icon"),)
    assert preview.absolute_favicon == (
        (baseurl + preview.favicon[0][0], None, "icon"),
    )

    # relative favicon from current path
    preview = link_preview(
        url=baseurl + "/article/333/",
        content=get_sample("favicon/relative2.html"),
    )
    assert preview.favicon == (("path/to/favicon.ico", None, "icon"),)
    assert preview.absolute_favicon == (
        (baseurl + "/article/333/" + preview.favicon[0][0], None, "icon"),
    )

    preview = link_preview(
        url=baseurl + "/article/444/index.html",
        content=get_sample("favicon/relative2.html"),
    )
    assert preview.favicon == (("path/to/favicon.ico", None, "icon"),)
    assert preview.absolute_favicon == (
        (baseurl + "/article/444/" + preview.favicon[0][0], None, "icon"),
    )

    # favicon placed out of page head tag, but still valid
    preview = link_preview(
        url=baseurl,
        content=get_sample("favicon/badplace.html"),
    )
    assert preview.favicon == (("/path/to/favicon.ico", None, "icon"),)

    # invalid rel attribute
    preview = link_preview(
        url=baseurl,
        content=get_sample("favicon/invalid-rel.html"),
    )
    assert preview.favicon == ()

    # empty rel attribute
    preview = link_preview(
        url=baseurl,
        content=get_sample("favicon/empty-rel.html"),
    )
    assert preview.favicon == ()

    # empty href attribute
    preview = link_preview(
        url=baseurl,
        content=get_sample("favicon/empty-href.html"),
    )
    assert preview.favicon == ()

    # duplicate
    preview = link_preview(
        url=baseurl,
        content=get_sample("favicon/duplicate.html"),
    )
    assert preview.favicon == (
        ("favicon.ico", None, "icon"),
        ("favicon.ico", None, "icon"),
    )

    preview = link_preview(
        url=baseurl,
        content=get_sample("favicon/duplicate2.html"),
    )
    assert preview.favicon == (
        ("favicon1.ico", None, "icon"),
        ("favicon2.ico", None, "icon"),
    )

    # sizes
    preview = link_preview(
        url=baseurl,
        content=get_sample("favicon/sizes.html"),
    )
    assert preview.favicon == (
        ("/apple-touch-icon.png", ((180, 180),), "apple-touch-icon"),
        ("/favicon-32x32.png", ((32, 32),), "icon"),
        ("/favicon-16x16.png", ((16, 16),), "icon"),
        ("/favicon.ico", ((32, 32), (16, 16)), "shortcut icon"),
        ("/favicon.ico", None, "icon"),
    )
    assert preview.absolute_favicon == (
        (baseurl + "/apple-touch-icon.png", ((180, 180),), "apple-touch-icon"),
        (baseurl + "/favicon-32x32.png", ((32, 32),), "icon"),
        (baseurl + "/favicon-16x16.png", ((16, 16),), "icon"),
        (baseurl + "/favicon.ico", ((32, 32), (16, 16)), "shortcut icon"),
        (baseurl + "/favicon.ico", None, "icon"),
    )



import pytest

from linkpreview import Link
from linkpreview.preview import Generic

from tests.helpers import get_sample


@pytest.mark.parametrize(
    "tin, tout",
    (
        ("empty.html", {"title": None, "description": None, "image": None}),
        (
            "title.html",
            {
                "title": "This title is at the title tag.",
                "description": None,
                "image": None,
            },
        ),
        (
            "h1-title.html",
            {
                "title": "This title is from the first h1 tag.",
                "description": None,
                "image": None,
            },
        ),
        (
            "meta-desc.html",
            {
                "title": None,
                "description": "This description is meta[name='description'].",
                "image": None,
            },
        ),
        (
            "h1-p-desc.html",
            {
                "title": "This is the first heading.",
                "description": "This is valid description.",
                "image": None,
            },
        ),
        (
            "p-desc.html",
            {
                "title": None,
                "description": "This description is from the first p.",
                "image": None,
            },
        ),
        (
            "h1-img.html",
            {
                "title": "This title is from the first h1 tag.",
                "description": None,
                "image": "http://localhost:8000/img/heck.jpg",
            },
        ),
        (
            "h1-img-far.html",
            {
                "title": "This title is from the first h1 tag.",
                "description": "Some blabla content",
                "image": "http://localhost:8000/img/heck1.jpg",
            },
        ),
        (
            "h1-img-far2.html",
            {
                "title": "This title is from the first h1 tag.",
                "description": "Some blabla content",
                "image": "http://localhost:8000/img/heck.jpg",
            },
        ),
        (
            "invalid-meta.html",
            {
                "title": "INVALID PAGE",
                "description": None,
                "image": None,
            },
        ),
        (
            "invalid-meta-syntax.html",
            {
                "title": "INVALID PAGE2",
                "description": "Jack",
                "image": None,
            },
        ),
        (
            "img-nosrc-invalid.html",
            {
                "title": None,
                "description": None,
                "image": None,
            },
        ),
        (
            "img-nosrc.html",
            {
                "title": None,
                "description": None,
                "image": "/uploads/animal/dog.png",
            },
        ),
    ),
)
def test_generic(tin, tout):
    link = Link("http://localhost", content=get_sample("generic/%s" % tin))
    preview = Generic(link, parser="html.parser")
    for key in tout.keys():
        assert getattr(preview, key) == tout[key]


@pytest.mark.parametrize(
    "tin, tout",
    (
        ("http://localhost", "localhost"),
        ("http://site.com", "site.com"),
        ("http://site.com:8080", "site.com"),
        ("http://root@site.com:8080", "site.com"),
        ("http://root:passwd@site.com:8080", "site.com"),
        ("http://root:passwd@a.b.site.com:8080", "a.b.site.com"),
        ("http://root:passwd@🍕.com:8080", "🍕.com"),
        ("http://root:passwd@xn--vi8h.com:8080", "xn--vi8h.com"),
        ("root:passwd@site.com:8080", ""),
    ),
)
def test_site_name(tin, tout):
    link = Link(tin, content="")
    preview = Generic(link, parser="html.parser")
    assert preview.site_name == tout



from linkpreview import Link, LinkGrabber, LinkPreview
from linkpreview.exceptions import InvalidMimeTypeError


def link_preview(
    url: str = None,
    content: str = None,
    parser: str = "html.parser",
):
    """
    Get link preview
    """
    if content is None:
        try:
            grabber = LinkGrabber()
            content, url = grabber.get_content(url)
        except InvalidMimeTypeError:
            content = ""

    link = Link(url, content)
    return LinkPreview(link, parser=parser)



import pytest

from linkpreview import Link
from linkpreview.preview import TwitterCard

from tests.helpers import get_sample


@pytest.mark.parametrize(
    "tin, tout",
    (
        (
            "available.html",
            {
                "title": "a title",
                "description": "لورم ایپزوم",
                "image": None,
                "site_name": None,
            },
        ),
        (
            "unavailable.html",
            {
                "title": None,
                "description": None,
                "image": None,
                "site_name": None,
            },
        ),
        (
            "with-image.html",
            {
                "title": "a title",
                "description": None,
                "image": "/img/heck.jpg",
                "site_name": None,
            },
        ),
    ),
)
def test_twitter_card(tin, tout):
    link = Link("http://localhost", content=get_sample("twittercard/%s" % tin))
    preview = TwitterCard(link, parser="html.parser")
    for key in tout.keys():
        assert getattr(preview, key) == tout[key]



from os.path import dirname

from pytest_httpserver import HTTPServer

from werkzeug.wrappers.response import Response

from linkpreview import link_preview

from tests.helpers import get_sample


def test_link_preview(httpserver: HTTPServer):
    httpserver.expect_request("/preview1").respond_with_data(
        get_sample("twittercard/with-image.html"),
        headers={"content-type": "text/html"},
    )
    httpserver.expect_request("/preview2").respond_with_data(
        get_sample("generic/h1-img.html"),
        headers={"content-type": "text/html"},
    )
    httpserver.expect_request("/preview-3.json").respond_with_data(
        "{}",
        headers={"content-type": "application/json"},
    )
    httpserver.expect_request("/redirected").respond_with_data(
        get_sample("generic/h1-img.html"),
        headers={"content-type": "text/html"},
    )
    redirected_url = "http://%s:%s/redirected" % (
        httpserver.host,
        httpserver.port,
    )
    httpserver.expect_request("/redirection").respond_with_response(
        Response(
            mimetype="text/html",
            headers={"location": redirected_url},
            status=301,
        )
    )

    url = httpserver.url_for("/preview1")
    preview = link_preview(url)
    assert preview.title == "a title"
    assert preview.force_title == "a title"
    assert preview.description is None
    assert preview.image == "/img/heck.jpg"
    assert preview.absolute_image == "%s%s" % (dirname(url), preview.image)
    assert preview.site_name == "localhost"

    url = httpserver.url_for("/preview2")
    preview = link_preview(url)
    assert preview.title == "This title is from the first h1 tag."
    assert preview.description is None
    assert preview.image == "http://localhost:8000/img/heck.jpg"
    assert preview.absolute_image == "http://localhost:8000/img/heck.jpg"
    assert preview.site_name == "localhost"

    preview = link_preview("https://example.com", content="OK")
    assert preview.title is None
    assert preview.description is None
    assert preview.image is None
    assert preview.absolute_image is None
    assert preview.force_title == "example.com"
    assert preview.site_name == "example.com"

    preview = link_preview("https://example.com/bird.jpg", content="OK")
    assert preview.force_title == "Bird"

    preview = link_preview("https://abc.com/the-bunny(720p).mkv", content="OK")
    assert preview.force_title == "The Bunny(720P)"

    preview = link_preview(
        "https://abc.com/the-bunny(720p)_season-1.mkv", content="OK"
    )
    assert preview.force_title == "The Bunny(720P) Season 1"

    preview = link_preview(
        "https://alex:123@abc.com/the-bunny(720p)", content="OK"
    )
    assert preview.force_title == "abc.com/the-bunny(720p)"
    assert preview.site_name == "abc.com"

    preview = link_preview("https://192.168.1.1", content="OK")
    assert preview.force_title == "192.168.1.1"
    assert preview.site_name == "192.168.1.1"

    preview = link_preview("https://192.168.1.1:9696", content="OK")
    assert preview.force_title == "192.168.1.1:9696"

    preview = link_preview(httpserver.url_for("/preview-3.json"))
    assert preview.title is None
    assert preview.description is None
    assert preview.image is None
    assert preview.absolute_image is None
    assert preview.force_title == "Preview 3"

    url = httpserver.url_for("/redirection")
    preview = link_preview(url)
    assert preview.link.url == redirected_url
    assert preview.title == "This title is from the first h1 tag."
    assert preview.description is None
    assert preview.image == "http://localhost:8000/img/heck.jpg"
    assert preview.absolute_image == "http://localhost:8000/img/heck.jpg"
    assert preview.to_dict() == {
        "site_name": "localhost",
        "title": "This title is from the first h1 tag.",
        "description": None,
        "image": "http://localhost:8000/img/heck.jpg",
        "absolute_image": "http://localhost:8000/img/heck.jpg",
        "force_title": "This title is from the first h1 tag.",
        "favicon": (),
        "absolute_favicon": (),
    }


def test_image():
    baseurl = "http://thesite.com"

    # absolute image
    preview = link_preview(
        url=baseurl + "/article/111",
        content=get_sample("generic/img-absolute.html"),
    )
    assert preview.image == "http://thesitescdn.com/uploads/animal/dog.png"
    assert preview.absolute_image == preview.image

    # relative image from root
    preview = link_preview(
        url=baseurl + "/article/222",
        content=get_sample("generic/img-relative.html"),
    )
    assert preview.image == "/uploads/animal/dog.png"
    assert preview.absolute_image == baseurl + preview.image

    # relative image from current path
    preview = link_preview(
        url=baseurl + "/article/333/",
        content=get_sample("generic/img-relative2.html"),
    )
    assert preview.image == "animal/dog.png"
    assert preview.absolute_image == baseurl + "/article/333/" + preview.image

    preview = link_preview(
        url=baseurl + "/article/444/index.html",
        content=get_sample("generic/img-relative2.html"),
    )
    assert preview.image == "animal/dog.png"
    assert preview.absolute_image == baseurl + "/article/444/" + preview.image



import pytest


from linkpreview import Link
from linkpreview.preview import JsonLd

from tests.helpers import get_sample


@pytest.mark.parametrize(
    "tin, tout",
    (
        (
            "available.html",
            {
                "title": "a title",
                "description": "لورم ایپزوم",
                "image": None,
                "site_name": "a title",
            },
        ),
        (
            "unavailable.html",
            {
                "title": None,
                "description": None,
                "image": None,
                "site_name": None,
            },
        ),
        (
            "with-image.html",
            {
                "title": "a title",
                "description": None,
                "image": "/img/heck.jpg",
                "site_name": None,
            },
        ),
        (
            "with-thumbnail.html",
            {
                "title": "a title",
                "description": None,
                "image": "/img/heck.jpg",
                "site_name": None,
            },
        ),
        (
            "invalid.html",
            {
                "title": None,
                "description": None,
                "image": None,
                "site_name": None,
            },
        ),
        (
            "article-website.html",
            {
                "title": "The Article",
                "description": None,
                "image": None,
                "site_name": "The Website",
            },
        ),
        (
            "website-article.html",
            {
                "title": "The Article",
                "description": None,
                "image": None,
                "site_name": "The Website",
            },
        ),
        (
            "sitename.html",
            {
                "title": "The Website",
                "description": None,
                "image": None,
                "site_name": "The Website",
            },
        ),
    ),
)
def test_jsonld(tin, tout):
    link = Link("http://localhost", content=get_sample("jsonld/%s" % tin))
    preview = JsonLd(link, parser="html.parser")
    for key in tout.keys():
        assert getattr(preview, key) == tout[key]



import pytest

from time import sleep

from pytest_httpserver import HTTPServer

from werkzeug.wrappers.response import Response

from requests.exceptions import ReadTimeout

from linkpreview import LinkGrabber
from linkpreview import exceptions

from tests.helpers import get_sample


def test_grabber(httpserver: HTTPServer):

    sample = get_sample("generic/h1-p-desc.html")

    class FakeResponse(Response):
        automatically_set_content_length = False

    httpserver.expect_request("/h1-p-desc").respond_with_data(
        sample, headers={"content-type": "text/html"}
    )
    httpserver.expect_request("/lazy").respond_with_handler(
        lambda x: sleep(0.6)
    )
    httpserver.expect_request("/huge").respond_with_response(
        FakeResponse(
            response=b"x" * 100000,
            mimetype="text/html",
        )
    )
    httpserver.expect_request("/badmime").respond_with_data(
        "{}", headers={"content-type": "application/json"}
    )
    httpserver.expect_request("/nomime").respond_with_response(
        Response(mimetype="")
    )
    httpserver.expect_request("/large").respond_with_response(
        FakeResponse(
            mimetype="text/html",
            headers={"content-length": "100000"},
        )
    )
    httpserver.expect_request("/redirected").respond_with_response(
        FakeResponse(
            mimetype="text/html",
            response=b"done!",
        )
    )
    redirected_url = "http://%s:%s/redirected" % (
        httpserver.host,
        httpserver.port,
    )
    httpserver.expect_request("/redirection").respond_with_response(
        FakeResponse(
            mimetype="text/html",
            headers={"location": redirected_url},
            status=301,
        )
    )
    httpserver.expect_request("/headers").respond_with_handler(
        lambda x: FakeResponse(
            response=x.headers["user-agent"].encode(),
            mimetype="text/html",
        )
    )

    # success
    grabber = LinkGrabber(maxsize=100)
    with pytest.raises(exceptions.MaximumContentSizeError):
        grabber.get_content(httpserver.url_for("/h1-p-desc"))

    # initial timeout
    grabber = LinkGrabber(initial_timeout=0.5)
    with pytest.raises(ReadTimeout):
        grabber.get_content(httpserver.url_for("/lazy"))

    # receive timeout
    grabber = LinkGrabber(receive_timeout=0.1, chunk_size=1)
    with pytest.raises(TimeoutError):
        grabber.get_content(httpserver.url_for("/huge"))

    # maxsize
    grabber = LinkGrabber(receive_timeout=10000, chunk_size=1024, maxsize=20)
    with pytest.raises(exceptions.MaximumContentSizeError):
        grabber.get_content(httpserver.url_for("/huge"))

    # large
    grabber = LinkGrabber(maxsize=100)
    with pytest.raises(exceptions.MaximumContentSizeError):
        grabber.get_content(httpserver.url_for("/large"))

    # nomime
    grabber = LinkGrabber()
    with pytest.raises(exceptions.InvalidContentError):
        grabber.get_content(httpserver.url_for("/nomime"))

    # badmime
    grabber = LinkGrabber()
    with pytest.raises(exceptions.InvalidContentError):
        grabber.get_content(httpserver.url_for("/badmime"))

    # redirection
    grabber = LinkGrabber()
    content, url = grabber.get_content(httpserver.url_for("/redirection"))
    assert url == redirected_url

    # default headers
    grabber = LinkGrabber()
    content, url = grabber.get_content(httpserver.url_for("/headers"))
    assert "Mozilla/5.0" in content.decode()

    # custom headers
    grabber = LinkGrabber()
    content, url = grabber.get_content(
        httpserver.url_for("/headers"),
        headers={"user-agent": "Googlebot"},
    )
    assert "Mozilla/5.0" not in content.decode()
    assert "Googlebot" in content.decode()



from bs4 import BeautifulSoup
from linkpreview.link import Link


class PreviewBase(object):  # pragma: nocover
    """
    Base for all web preview.
    """

    def __init__(self, link: Link, parser: str):
        self.link = link
        self._soup = BeautifulSoup(self.link.content, parser)

    @property
    def title(self):
        raise NotImplementedError

    @property
    def description(self):
        raise NotImplementedError

    @property
    def image(self):
        raise NotImplementedError



from os.path import dirname, abspath, join

this_dir = dirname(abspath(__file__))
stuff_dir = join(this_dir, "stuff")


def get_sample(path):
    with open(join(stuff_dir, path), "r") as f:
        return f.read()
